import requests
from magik.constants import API_BASE_URL
from magik.config import get_magik_api_key


def logChatResponse(prompt_slug, prompt_sent, model, chat_completion, testId):
    """
    Track the request and response.
    """
    requests.post(
        f"{API_BASE_URL}/api/v1/savePromptAndResponse",
        json={
            "promptSlug": prompt_slug,
            "promptSent": prompt_sent,
            "languageModelId": model,
            "completion": chat_completion,
            "testId": testId,
        },
        headers={
            "magik-api-key": get_magik_api_key(),
        },
    )



from magik.decorators import magik_eval
from magik.evaluators import (
    contains_link,
)


# Define a custom test like this
#
# output_to_test is automatically passed in when we run the test
# as long as we have the @magik_eval decorator
#
@magik_eval
def is_valid_length(output_to_test):
    result = len(output_to_test) > 1000
    return {
        "result": result,
        "reason": "Output length is correct" if result else "Output length is invalid",
    }


# Every test must contain:
# - description: a description of the test
# - eval: the function to run to evaluate the output
# - prompt_vars:
#   - dictionary of variable values to pass into the prompt
#   - these will replace variables in curly {braces} in your prompt.txt file in local tests
# - failure_labels: the labels to tag the failure with - used for the analytics dashboard
tests = [
    {
        "description": "output contains a link",
        "eval": contains_link(),
        "prompt_vars": {
            "app_name": "Magik",
        },
        "failure_labels": ["bad_response_format"],
    },
    {
        "description": "output passes custom function",
        "eval": is_valid_length(),
        "prompt_vars": {
            "app_name": "Magik",
        },
        "failure_labels": ["custom_failure"],
    },
]



# Normalize the output to a testable format
def normalize_output_string(str):
    str = str.replace("\n", "")
    str = str.replace("\t", "")
    str = str.replace("'", "")
    str = str.replace('"', "")
    return str


def substitute_vars(string, vars_dict):
    return string.format(**vars_dict)


def standardize_url(url):
    if url.startswith("http://") or url.startswith("https://"):
        return url
    else:
        return "http://" + url


def generate_grading_prompt(output_to_evaluate, grading_criteria):
    return (
        """
    You are grading a response string according to a grading criteria given to you.
    
    If the gradient criteria is true, then the test_result is True. Otherwise, it is false.
    
    Explain the reason for the test_result in the reason field.
    
    Respond with a python dictionary object that looks like this:
    {test_result: boolean; reason: string;}
    
    Examples:

    String: Qui est le président des États-Unis?
    Grading Criteria: Contains the french words for 'president' and 'United States'
    {"test_result": True, "reason": "the content contains the word 'world'"}

    String: Earth is the second planet from the sun.
    Grading Criteria: If the fact is true, then the test_result is True. Otherwise, it is false.
    {"test_result": False, "reason": "The string is factually inaccurate - Earth is actually the third planet from the sun."}`
    """
        + f"""
        String: {output_to_evaluate}
        Grading Criteria: {grading_criteria}
    """
    )



#!/usr/bin/env python3

import argparse
from magik.initialize import initialize
from magik.generate import generate_test
from magik.deploy import deploy_test
from magik.internal_logger import logger
from magik.run import Run


def main():
    parser = argparse.ArgumentParser(
        prog="magik", description="Magik testing suite CLI tool"
    )

    subparsers = parser.add_subparsers(title="commands", dest="command")

    # magik init
    parser_init = subparsers.add_parser("init", help="Initialize magik testing suite")
    parser_init.set_defaults(func=init)

    # magik generate <test-name>
    parser_generate = subparsers.add_parser("generate", help="Generate a new test")
    parser_generate.add_argument("test_name", help="Name of the test")
    parser_generate.set_defaults(func=generate)

    # magik run <test-name>
    parser_run = subparsers.add_parser("run", help="Run a test")
    parser_run.add_argument("test_name", help="Name of the test")
    parser_run.set_defaults(func=run)

    # magik deploy <test-name>
    parser_deploy = subparsers.add_parser("deploy", help="Deploy a test")
    parser_deploy.add_argument("test_name", help="Name of the test")
    parser_deploy.set_defaults(func=deploy)

    # magik run_prod --prod --test <test-name> --prompt <prompt-slug> --start_date <start-date> --end_date <end_date>
    parser_run_production = subparsers.add_parser(
        "run_prod", help="Run a test in production"
    )
    parser_run_production.add_argument(
        "--test", dest="test_name", help="Name of the test", required=False, default="*"
    )
    parser_run_production.add_argument(
        "--prompt", dest="prompt_slug", help="Slug of the prompt", required=True
    )
    parser_run_production.add_argument("--start_date", help="Start date of the test")
    parser_run_production.add_argument("--end_date", help="End date of the test")
    parser_run_production.set_defaults(func=run_prod)

    args = parser.parse_args()
    args.func(args)


def init(args):
    initialize()


def generate(args):
    generate_test(args.test_name)


def run(args):
    test_runner = Run()
    test_runner.run_tests(args.test_name)


def deploy(args):
    deploy_test(args.test_name)


def run_prod(args):
    test_runner = Run()
    test_runner.run_tests_in_prod(
        start_date=args.start_date,
        end_date=args.end_date,
        prompt_slug=args.prompt_slug,
        test_slug=args.test_name,
    )


if __name__ == "__main__":
    main()



def magik_eval(func):
    def wrapper(*args, **kwargs):
        wrapped_func = lambda output_to_test: func(
            *args, **kwargs, output_to_test=output_to_test
        )

        # Convert each value to a string and join them with commas, with strings wrapped in quotes
        arg_string = ", ".join(
            f"'{value}'"
            if isinstance(value, str)
            else value.__name__
            if callable(value)
            else str(value)
            for value in args
        )

        wrapped_func.__name__ = f"{func.__name__}({arg_string})"
        return wrapped_func

    return wrapper



import os
import requests
import sys
import importlib.util
from datetime import datetime
from magik.internal_logger import logger
from magik.utils import substitute_vars
from magik.openai_helper import OpenAI
from magik.sys_exec import read_from_file, create_file
from magik.config import get_open_ai_default_model, get_magik_api_key
from magik.constants import TESTRUNS_DIR, TEST_DIR, RUN_URL


class Run:
    def __init__(self):
        self.saved_prompt_response = ""

    def run_tests(self, test_name):
        tests = self._load_tests(test_name)
        raw_prompt = self._load_prompt(test_name)
        log_file_path = self._log_file_path(test_name)
        self._run_tests_for_prompt(
            test_name, tests, raw_prompt, log_file_path=log_file_path
        )

    def run_tests_in_prod(self, start_date, end_date, prompt_slug, test_slug):
        request_data = {
            "source": "CLI",
            "start_date": start_date,
            "end_date": end_date,
            "prompt_tests": [
                {
                    "prompt_slug": prompt_slug,
                    "test_slugs": "*" if test_slug == "*" else test_slug.split(","),
                }
            ],
        }
        # Remove None fields from the payload
        payload = {k: v for k, v in request_data.items() if v is not None}

        logger.debug(f"Sending request to {RUN_URL} with data: {request_data}\n")
        response = requests.post(
            RUN_URL,
            json=payload,
            headers={
                "magik-api-key": get_magik_api_key(),
            },
        )
        if response.status_code != 200:
            logger.error(f"ERROR: Failed to trigger test in prod: {response.text}")
            return

    def _run_tests_for_prompt(self, test_name, tests, raw_prompt, log_file_path):
        self._log_to_file_and_console("---------------")
        self._log_to_file_and_console("TEST RESULTS")
        self._log_to_file_and_console("---------------")
        self._log_to_file_and_console("\n")

        num_tests_passed = 0
        for test in tests:
            test_result = self._run_individual_test_for_prompt(
                test_name, test, raw_prompt, log_file_path
            )
            if test_result:
                num_tests_passed += 1

        num_tests_failed = len(tests) - num_tests_passed

        logger.info("\n------------")
        if num_tests_passed == len(tests):
            logger.info("✅ All tests passed")
            logger.info("\n\n")
        else:
            logger.info(
                f"""
    ✅ {num_tests_passed}/{len(tests)} tests passed
    ❌ {num_tests_failed}/{len(tests)} tests failed
            """
            )

    def _run_individual_test_for_prompt(
        self, test_name, test, raw_prompt, log_file_path
    ):
        openai = OpenAI()
        prompt_vars = test["prompt_vars"]
        model = get_open_ai_default_model()
        if len(prompt_vars) == 0:
            prompt = raw_prompt
            if len(self.saved_prompt_response) == 0:
                self.saved_prompt_response = openai.openai_chat_completion_message(
                    model=model, prompt=prompt
                )
            prompt_response = self.saved_prompt_response
        else:
            prompt = substitute_vars(raw_prompt, prompt_vars)
            prompt_response = openai.openai_chat_completion_message(
                model=model, prompt=prompt
            )

        return self._run_individual_test_for_prompt_response(
            test_name, test, prompt, prompt_response, log_file_path=log_file_path
        )

    def _run_individual_test_for_prompt_response(
        self, test_name, test, prompt, prompt_response, log_file_path
    ):
        test_function_result_obj = test["eval"](prompt_response)
        result_obj = self._generate_result_object(
            test=test,
            test_result=test_function_result_obj,
            prompt=prompt,
            prompt_response=prompt_response,
        )

        self._log_results(
            result_obj,
            log_file_path=log_file_path,
        )

        return result_obj["test_result"]["result"]

    def _load_prompt(self, test_name):
        test_file_path = f"{TEST_DIR}/{test_name}/prompt.txt"
        absolute_path = os.path.abspath(test_file_path)
        return read_from_file(absolute_path)

    def _load_tests(self, test_name):
        test_file_path = f"{TEST_DIR}/{test_name}/assertions.py"
        # Get the absolute path by resolving against the current working directory
        absolute_path = os.path.abspath(test_file_path)

        # Get the directory path and module name from the absolute path
        directory, module_name = os.path.split(absolute_path)
        module_name = os.path.splitext(module_name)[0]

        # Load the module dynamically
        spec = importlib.util.spec_from_file_location(module_name, absolute_path)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)

        # Access the `tests` array from the module
        return getattr(module, "tests", [])

    def _generate_result_object(self, test, test_result, prompt, prompt_response):
        did_test_pass = test_result["result"]
        failure_labels = test["failure_labels"] if not did_test_pass else []
        test_function_name = test["eval"].__name__
        return {
            "test": {
                **test,
                "eval": test_function_name,
            },
            "test_result": test_result,
            "prompt": prompt,
            "prompt_response": prompt_response,
            "failure_labels": failure_labels,
        }

    def _log_results(self, result_obj, log_file_path=None):
        sys.stdout.flush()  # Flush the stdout buffer to ensure immediate printing to the console
        test_description = result_obj["test"]["description"]
        if log_file_path is not None:
            create_file(log_file_path)
            with open(log_file_path, "a") as log_file:
                self._log_to_file_and_console(
                    f"Test: {test_description}", log_file, color="cyan"
                )
                self._log_to_file_and_console(f"-----", log_file, color="cyan")
                self._log_prompt_results(result_obj, log_file)
                self._log_test_results(result_obj, log_file)
        else:
            self._log_to_file_and_console(f"Test: {test_description}", color="cyan")
            self._log_to_file_and_console(f"-----", color="cyan")
            self._log_prompt_results(result_obj)
            self._log_test_results(result_obj)

    def _log_to_file_and_console(self, output, log_file=None, color=None):
        if log_file is not None:
            log_file.write(output + "\n")
            log_file.flush()  # Ensure immediate writing to the file

        if color is not None:
            logger.log_with_color(output, color)
        else:
            logger.info(output)

    def _log_prompt_results(self, result_obj, log_file=None):
        prompt = result_obj["prompt"]
        prompt_response = result_obj["prompt_response"]
        self._log_to_file_and_console(f"Prompt: {prompt}\n", log_file)
        self._log_to_file_and_console(f"Prompt Response: {prompt_response}\n", log_file)

    def _log_test_results(self, result_obj, log_file=None):
        test_function_result_bool = result_obj["test_result"]["result"]
        test_result_str = "✅ Passed" if test_function_result_bool else "❌ Failed"
        test_result_reason = result_obj["test_result"]["reason"]
        failure_labels = result_obj["failure_labels"]

        self._log_to_file_and_console(f"Test Result: {test_result_str}", log_file)
        self._log_to_file_and_console(f"Reason: {test_result_reason}", log_file)
        self._log_to_file_and_console(f"Failure Labels: {failure_labels}", log_file)
        self._log_to_file_and_console("\n", log_file)

    def _log_file_path(self, test_name):
        current_timestamp = datetime.now()
        formatted_timestamp = current_timestamp.strftime("%Y-%m-%d_%H-%M-%S")
        log_file_path = f"{TESTRUNS_DIR}/{test_name}/{formatted_timestamp}.txt"
        return log_file_path



import openai
from magik.constants import API_BASE_URL
from magik.config import (
    get_open_ai_api_key,
    get_open_ai_default_model,
    get_magik_api_key,
)


class OpenAI:
    def __init__(self):
        self.openai = openai
        self.default_model = get_open_ai_default_model()
        self.magik_api_key = get_magik_api_key()
        self.openai.api_key = get_open_ai_api_key()

    def openai_chat_completion(self, model, prompt):
        """
        Call the OpenAI API to generate a chat completion for models like GPT 3.5 turbo and GPT 4
        """
        return self.openai.ChatCompletion.create(
            model=model, messages=[{"role": "user", "content": prompt}]
        )

    def openai_chat_completion_message(self, model, prompt):
        """
        Call the OpenAI API to generate a chat completion for models like GPT 3.5 turbo and GPT 4
        Returns just the message content string
        """
        response = self.openai_chat_completion(model, prompt)
        return response.choices[0].message.content
